/* 
 * Copyright (c) 2010 ISPRAS
 *
 * Institute for System Programming of the Russian Academy of Sciences (ISPRAS)
 * 25 A.Solzhenitsyn st. Moscow 109004 Russia
 * http://hardware.ispras.ru
 *
 * All rights reserved.
 *
 * This source code is free software; you can redistribute it
 * and/or modify it in source code form under the terms of the GNU
 * General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 * 
 * $Id: vpi_systf.c,v 1.2 2010/03/04 15:40:47 kamkin Exp $
 */

#include <assert.h>
#include <ctype.h>
#include <stdio.h>
#include <string.h>

#include "common.h"
#include "vpi_systf.h"

#include "../error.h"

static void vt_print_function_vpi_systf_c(FILE *file, char *name, char *template, vt_options_t *options)
{
    if(template == NULL)
    {
    	fprintf(file, "static int calltf_%s(char *user_data)\n{\n", name);
        fprintf(file, "    // TODO: Add a call of your function here\n");
        fprintf(file, "    return 0;\n}\n\n");
    }
    else
    {
    	fprintf(file, "static int calltf_%s(char *user_data)\n{\n", name);
    	fprintf(file, template);
    	fprintf(file, "}\n\n");
    }
}

static void vt_print_register_vpi_systf_c(FILE *file, char *name, vt_options_t *options)
{
    int i;
    char big_name[strlen(name) + 1];

    for(i = 0; name[i]; i++)
    	{ big_name[i] = toupper(name[i]); }
    	
    big_name[i] = name[i];

    fprintf(file, "static void register_calltf_%s(void)\n{\n", name);
    fprintf(file, "    s_vpi_systf_data tf_data;\n");
    fprintf(file, "    #ifdef ICARUS_VERILOG\n");
    fprintf(file, "        vpiHandle tf_handle;\n");
    fprintf(file, "    #endif\n\n");
    fprintf(file, "    tf_data.type        = vpiSysTask;\n");
    fprintf(file, "    tf_data.sysfunctype = 0;\n");
    fprintf(file, "    tf_data.tfname      = TF_NAME_%s;\n", big_name);
    fprintf(file, "    tf_data.calltf      = calltf_%s;\n", name);
    fprintf(file, "    tf_data.compiletf   = NULL;\n");
    fprintf(file, "    tf_data.sizetf      = NULL;\n");
    fprintf(file, "    tf_data.user_data   = NULL;\n\n");
    fprintf(file, "    #ifdef ICARUS_VERILOG\n");
    fprintf(file, "        vpi_register_systf(&tf_data);\n");
    fprintf(file, "    #else\n");
    fprintf(file, "        tf_handle = vpi_register_systf(&tf_data);\n");
    fprintf(file, "        vpi_free_object(tf_handle);\n");
    fprintf(file, "    #endif\n}\n\n");
}

void vt_print_vpi_systf_c(FILE *file, vt_design_t *design, vt_options_t *options)
{
    assert(file != NULL);
    assert(design != NULL);
    assert(options != NULL);

    fprintf(file, "#include \"%s\"\n\n", options->vpi_systf_h);
    
    vt_print_decorated_comment(file, "VPI Tasks and Functions", options);

    vt_print_function_vpi_systf_c(file, "set_scenario", VPI_SYSTF_TEMPLATE, options);
    vt_print_function_vpi_systf_c(file, "apply_action", NULL, options);
    vt_print_function_vpi_systf_c(file, "check_action", NULL, options);

    vt_print_decorated_comment(file, "Registration of VPI Tasks and Functions", options);

    vt_print_register_vpi_systf_c(file, "set_scenario", options);
    vt_print_register_vpi_systf_c(file, "apply_action", options);
    vt_print_register_vpi_systf_c(file, "check_action", options);

    fprintf(file, "void (*vlog_startup_routines[])(void) =\n{\n");
    fprintf(file, "    register_calltf_set_scenario,\n");
    fprintf(file, "    register_calltf_apply_action,\n");
    fprintf(file, "    register_calltf_check_action,\n");
    fprintf(file, "    NULL\n};\n\n");
}

void vt_print_vpi_systf_h(FILE *file, vt_design_t *design, vt_options_t *options)
{
    assert(file != NULL);
    assert(design != NULL);
    assert(options != NULL);

    fprintf(file, "#ifndef _VPI_SYSTF_H_INCLUDED_\n");
    fprintf(file, "#define _VPI_SYSTF_H_INCLUDED_\n\n");
    
    fprintf(file, "#include \"vsim/vsim.h\"\n\n");
    
    fprintf(file, "#define TF_NAME_SET_SCENARIO       \"$setScenario\"\n");
    fprintf(file, "#define TF_NAME_APPLY_ACTION       \"$applyAction\"\n");
    fprintf(file, "#define TF_NAME_CHECK_ACTION       \"$checkAction\"\n\n");

    vt_print_decorated_comment(file, "Registration of VPI Tasks and Functions", options);
    
    fprintf(file, "extern void (*vlog_startup_routines[])(void);\n\n");
    fprintf(file, "#endif // _VPI_SYSTF_H_INCLUDED_\n\n");
}
